package com.parseSystem.rabbitmq;

import com.alibaba.fastjson.JSONObject;
import com.parseSystem.event.EventHandlerService;
import com.parseSystem.kafka.constant.kafkaConstants;
import com.parseSystem.utils.DataQueueManager;
import com.parseSystem.utils.ParseUtil;
import com.parseSystem.vehicle.VehicleData;
import com.rabbitmq.client.Channel;
import lombok.extern.log4j.Log4j2;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.apache.kafka.clients.consumer.ConsumerRecords;
import org.apache.kafka.clients.consumer.KafkaConsumer;
import org.apache.kafka.common.TopicPartition;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.Queue;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.time.Duration;
import java.util.Collections;

/**
 * 监听车辆事件变更消费队列
 *
 * @author 冯凯
 * @version 1.0
 * @date 2023/11/27 22:19
 */
@Component
@Log4j2
public class ListenEventChangeRabbitMq {

    @Autowired
    private DataQueueManager dataQueueManager;

    @Autowired
    private KafkaConsumer<String, String> consumer;

    @Autowired
    private EventHandlerService eventHandlerService;

    /**
     * RabbitMQ消费者，订阅"kafka_top"队列中的消息并处理
     *
     * @param mesg    接收到的消息内容
     * @param message RabbitMQ中的Message对象
     * @param channel RabbitMQ中的Channel对象
     */
    @RabbitListener(queuesToDeclare = {@Queue(value = "kafka_top")})
    public void consumerSubscribe(String mesg, Message message, Channel channel) {
        log.info("收到准备订阅主题:{}", mesg);

        // 将接收到的消息解析为kafkaConstants对象
        kafkaConstants kafkaConstants = JSONObject.parseObject(mesg, kafkaConstants.class);

        // 创建TopicPartition对象
        TopicPartition topicPartition = new TopicPartition(kafkaConstants.getTopic(), Integer.valueOf(kafkaConstants.getPartition()));

        try {
            // 手动确认接收到的消息
            channel.basicAck(message.getMessageProperties().getDeliveryTag(), false);

            // 将消费者订阅的主题设置为"kafka_top"队列的第一个分区
            consumer.assign(Collections.singleton(topicPartition));
            // 开启线程持续拉取数据
            while (true) {
                ConsumerRecords<String, String> records = null;
                try {
                    // 每个一秒钟拉取一次数据
                    records = consumer.poll(Duration.ofMillis(1000));
                    for (ConsumerRecord<String, String> record : records) {
                        System.out.println(record.value());
                        long startTime = System.currentTimeMillis();
                        log.info("开始消费时间{}:", startTime);
                        // 解析数据
                        String data = ParseUtil.sixteenToStr(record.value());
                        // 构建数据对象
                        VehicleData vehicleData = VehicleData.getBuild(data);
                        log.info(vehicleData);
                        dataQueueManager.enqueueData(vehicleData);
                        // 调用执行事件
                        eventHandlerService.executeEvent(vehicleData);
                        log.info("耗费时间{}", System.currentTimeMillis() - startTime);
                    }
                } catch (Exception e) {
                    log.info("records： {}", records);
                    log.error(e);
                }
            }


            // 订阅特定分区

        } catch (IOException e) {
            throw new RuntimeException(e);
        }

    }


}
